<?php

namespace Drupal\frontend_publishing\Service;

use Drupal\Core\Entity\ContentEntityBase;

/**
 *
 */
class StateChange {

  /**
   * Publish a page and all rows, components and contents within.
   */
  public function publish(ContentEntityBase &$entity) {
    $entity->setRevisionLogMessage($message);
    $handlers = \Drupal::service('plugin.manager.frontend_publishing_handler')->getHandlers();
    foreach ($handlers as $handler) {
      $handler->publish($entity);
    }
  }

  /**
   * Publish a page and all rows, components and contents within.
   *
   * Returns a tree array of all published node and revision ids.
   *
   * @return The published node and revision ids.
   */
  public function transition(ContentEntityBase &$entity, $message, $state) {
    $entity->setRevisionLogMessage($message);
    $moduleHandler = \Drupal::service('module_handler');
    if ($moduleHandler->moduleExists('content_moderation') && \Drupal::service('content_moderation.moderation_information')->isModeratedEntity($entity)) {
      $entity->moderation_state->value = $state;
      $entity->setNewRevision();
      $handlers = \Drupal::service('plugin.manager.frontend_publishing_handler')->getHandlers();
      foreach ($handlers as $handler) {
        $handler->transition($entity);
      }
    }
    else {
      if ($state == 'publish') {
        $this->publish($entity);
      }
      elseif ($state == 'unpublish') {
        $this->unpublish($entity);
      }
    }
  }

  /**
   * Publish a page and all rows, components and contents within.
   *
   * Returns a tree array of all published node and revision ids.
   *
   * @return The published node and revision ids.
   */
  public function unpublish(ContentEntityBase &$entity, $message) {
    $entity->setRevisionLogMessage($message);
    $handlers = \Drupal::service('plugin.manager.frontend_publishing_handler')->getHandlers();
    foreach ($handlers as $handler) {
      $handler->unpublish($entity);
    }
  }

  /**
   * Clones a entity and passes each language version to the registered handlers.
   * Returns the cloned entity.
   *
   * @param \Drupal\Core\Entity\ContentEntityBase $entity
   *   The entity to clone.
   *
   * @return \Drupal\Core\Entity\ContentEntityBase The cloned entity in the default translation.
   */
  public function copy(ContentEntityBase &$entity) {
    $clone = NULL;
    $handlers = \Drupal::service('plugin.manager.frontend_publishing_handler')->getHandlers();
    foreach ($handlers as $handler) {
      if ($clone != NULL) {
        $clone = $clone->getUntranslated();
        $entity = $entity->getUntranslated();
        $clone = $handler->copy($entity, $clone);
        foreach ($clone->getTranslationLanguages(FALSE) as $language) {
          $clone = $clone->getTranslation($language->getId());
          $entity = $entity->getTranslation($language->getId());
          $clone = $handler->copy($entity, $clone);
        }
      }
      else {
        $clone = $handler->copy($entity, $clone);
      }
    }
    foreach ($clone->getTranslationLanguages(TRUE) as $language) {
      $clone->getTranslation($language->getId())->save();
    }
    $clone = $clone->getUntranslated();
    return $clone;
  }

}
